#include <FastLED.h>

// -- Settings -- //
const int NUMBER_OF_STRIPS = 2;
const int NUM_LEDS_PER_STRIP = 26; // Standard: 26, Bass: 33, Yamaha: 27

const int LED_PIN_LEFT = 2;
const int LED_PIN_RIGHT = 3;

const bool POTI = true; // Kein Poti an Lukas Gitarren
const int POTI_PIN = A0;
const int GLOBAL_BRIGHTNESS = 180; // Nur relevant, wenn POTI = false

const int SELECTOR_PINS[] = {4, 5, 6, 7, 8}; // Kein fünfter Effekt auf D8 an Lukas Gitarren
const int NUMBER_OF_SELECTOR_PINS = 5;

#define LED_TYPE NEOPIXEL

#define FX_1 fx_strobe_1
#define FX_2 fx_strobe_2
#define FX_3 fx_strobe_3
#define FX_4 fx_strobe_4
#define FX_5 fx_strobe_5
// -------------- //

const int NUM_LEDS_TOTAL = NUM_LEDS_PER_STRIP * NUMBER_OF_STRIPS;

CRGB leds_R[NUM_LEDS_PER_STRIP];
CRGB leds_L[NUM_LEDS_PER_STRIP];

int selector_position;
int poti_value;

void setup()
{
  for (int selector_pin = 0; selector_pin < NUMBER_OF_SELECTOR_PINS; selector_pin++)
  {
    pinMode(SELECTOR_PINS[selector_pin], INPUT_PULLUP);
  }

  FastLED.addLeds<LED_TYPE, LED_PIN_RIGHT>(leds_R, NUM_LEDS_PER_STRIP).setCorrection(TypicalLEDStrip);
  FastLED.addLeds<LED_TYPE, LED_PIN_LEFT>(leds_L, NUM_LEDS_PER_STRIP).setCorrection(TypicalLEDStrip);

  // FastLED.setMaxPowerInVoltsAndMilliamps(3, 200);
  // FastLED.setMaxPowerInMilliWatts(500);
}

void loop()
{
  read_selector();
  update_state();
  set_brightness();
  FastLED.show();
}

typedef void (*SimplePatternList[])();
SimplePatternList FX_LIST = {FX_1, FX_2, FX_3, FX_4, FX_5};

void read_selector()
{
  for (int selector_pin = 0; selector_pin < NUMBER_OF_SELECTOR_PINS; selector_pin++)
  {
    int selector_pin_state = (digitalRead(SELECTOR_PINS[selector_pin]));

    if (selector_pin_state == LOW)
    {
      selector_position = selector_pin;
    };
  }
}

void update_state()
{
  static int state;
  if (selector_position != state)
  {
    FastLED.clear();
  }
  FX_LIST[selector_position]();
  state = selector_position;
}

void set_brightness()
{
  static int brightness;
  if (POTI == true)
  {
    poti_value = analogRead(POTI_PIN);
    brightness = map(poti_value, 0, 1023, 255, 0);
  }
  else
  {
    brightness = GLOBAL_BRIGHTNESS;
  }
  FastLED.setBrightness(brightness);
}
